/* cairot.h

  The Cairo Toolkit, a very tiny cross-platform toolkit for making
  Cairo demo and test programs.

  Copyright (c) 2010 Bill Spitzak (spitzak@gmail.com)

  All code is based on FLTK2.0.

  License is GPL2 or later, plus a linking exception that says you can
  use this code unchanged in any program no matter what the license of
  that program is. However if you modify this code then you must
  include the modified version as source code under this same license
  or (by removing the linking exception) under the GPL or LGPL.

  Please check with www.fltk.org for license details. However the
  intention is to allow this source code to be used for any purpose,
  as long as you don't "close" this source code, without propagating
  the restrictions to other code.
*/

#ifndef CAIROT_H
#define CAIROT_H

//! Id for windows, this is the same number used by operating system
typedef unsigned cairot_window_t;

//! Make a new window with given size and return it's id
cairot_window_t cairot_create_window(int width, int height);

// NYI: cairot_destroy_window

//! Change the text in the titlebar of the window, name is UTF-8
void cairot_set_window_name(cairot_window_t, const char* name);

// NYI: cairot_get_window_name, probably not needed

#define CAIROT_STATE_MAPPED 1 //!< window is visible
// NYI: any state other than mapped, such as maximized or iconized or unmapped!

//! Change the state of the window to set of CAIROT_STATE bitflags
void cairot_set_window_state(cairot_window_t, unsigned state);

// NYI: cairot_get_window_state

// NYI: cairot_set_window_size

void cairot_get_window_size(cairot_window_t, int* width, int* height);

//! Indicate that draw(window) must be called
void cairot_redraw(cairot_window_t);

typedef struct _cairo cairo_t; // don't include cairo.h

//! Return cairo_t that can be used to draw into window
cairo_t* cairo_for(cairot_window_t);

//! Set function to call to draw windows
void cairot_set_draw(void(*)(cairot_window_t));

//! Set function to call when windows are moved or resized
void cairot_set_reshape(void(*)(cairot_window_t, int x, int y, int w, int h));

//! Set function to call when user closes the window
void cairot_set_close(void(*)(cairot_window_t));

//! Set function to call when user presses a mouse button
void cairot_set_push(void(*)(cairot_window_t, int button, int x, int y));

//! Set function to call when user drags mouse
void cairot_set_drag(void(*)(cairot_window_t, int button, int x, int y));

//! Set function to call when user releases mouse
void cairot_set_release(void(*)(cairot_window_t, int button, int x, int y));

//! Set function for mouse movement, window == 0 for other programs
void cairot_set_motion(void(*)(cairot_window_t, int x, int y));

// NYI: mouse wheel

//! Set function for keyboard focus change, window == 0 for other programs
void cairot_set_focus(void(*)(cairot_window_t));

//! Set function for keystrokes
void cairot_set_keypress(void(*)(cairot_window_t, int key, const char* text));

// NYI: keyrelease

// NYI: keyrepeat

// NYI: callback after time for animation

//! Handle events until somebody calls exit()
int cairot_run();

#endif
