#include "cairot.h"
#include <cairo/cairo.h>
#include <math.h>
#include <stdio.h>

static char text0[100] = "";
static char text1[100] = "focus out";
static char text2[100] = "no keystrokes";

static cairot_window_t window;
static void redraw_window() {cairot_redraw(window);}

void draw(cairot_window_t window)
{
  int W, H; cairot_get_window_size(window, &W, &H);
  cairo_t* cr = cairo_for(window);

  /* erase the window */
  cairo_set_source_rgb(cr, 1,1,1);
  cairo_rectangle(cr, 0, 0, W, H);
  cairo_fill(cr);

  cairo_select_font_face (cr, "Sans",
			  CAIRO_FONT_SLANT_NORMAL,
			  CAIRO_FONT_WEIGHT_NORMAL);
  double height = 24.0;

  cairo_set_font_size (cr, height);

  cairo_set_source_rgb(cr, 0,0,0);
  cairo_move_to(cr, 10, (H-height)/2);
  cairo_show_text(cr, text0);
  cairo_move_to(cr, 10, (H+height)/2);
  cairo_show_text(cr, text1);
  cairo_move_to(cr, 10, (H+3*height)/2);
  cairo_show_text(cr, text2);
}

void push(cairot_window_t window, int button, int X, int Y) {
  sprintf(text0, "button %d pushed at %d %d", button, X, Y);
  redraw_window();
}

void drag(cairot_window_t window, int button, int X, int Y) {
  sprintf(text0, "button %d drag to %d %d", button, X, Y);
  redraw_window();
}

void release(cairot_window_t window, int button, int X, int Y) {
  sprintf(text0, "button %d release at %d %d", button, X, Y);
  redraw_window();
}

void motion(cairot_window_t window, int X, int Y) {
  if (window)
    sprintf(text0, "move to %d %d", X, Y);
  else
    sprintf(text0, "move out");
  redraw_window();
}

void focus(cairot_window_t window) {
  sprintf(text1, window ? "focus in" : "focus out");
  redraw_window();
}

void keypress(cairot_window_t window, int keysym, const char* text) {
  if (text[0]>=' ' && text[0]!=127)
    sprintf(text2, "'%s' typed", text);
  else
    sprintf(text2, "key %#x typed", keysym);
  redraw_window();
}

int main(int argc, char** argv)
{
  cairot_set_draw(draw);
  cairot_set_push(push);
  cairot_set_drag(drag);
  cairot_set_release(release);
  cairot_set_motion(motion);
  cairot_set_focus(focus);
  cairot_set_keypress(keypress);
  window = cairot_create_window(300, 300);
  cairot_set_window_name(window, argv[0]);
  cairot_set_window_state(window, CAIROT_STATE_MAPPED);
  return cairot_run();
}
