#include "cairot.h"
#include <cairo/cairo.h>
#include <math.h>
#include <stdio.h>
#include <malloc.h>

#define WIDTH 240
#define HEIGHT 240
#define CHECKER 5
#define YCHECKER 5

cairo_surface_t* makeimage()
{
  int x, y;
  int w = cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32, WIDTH);
  unsigned* data = malloc(w * HEIGHT);
  for (y = 0; y < HEIGHT; ++y)
    for (x = 0; x < WIDTH; ++x) {
      unsigned g = x*127/WIDTH;
      unsigned b = y*127/HEIGHT;
      unsigned p = 0xff000000 + (g<<8) + b;
      if ((x%(2*CHECKER) < CHECKER) ^ (y%(2*YCHECKER) < YCHECKER))
        p += 0xFF8080;
      data[y*w/4 + x] = p;
    }
  return cairo_image_surface_create_for_data
    ( (unsigned char*)data, CAIRO_FORMAT_ARGB32, WIDTH, HEIGHT, w);
}

cairo_pattern_t* pattern;

int ox = 0;
int oy = 0;
float scale = 1.0;
float rotate = 0.0;

void draw(cairot_window_t window)
{
  int W, H; cairot_get_window_size(window, &W, &H);
  cairo_t* cr = cairo_for(window);

  cairo_save(cr);

  cairo_rectangle(cr, 0, 0, W, H);
  /* cairo_surface_set_device_scale(cairo_get_target(cr), 4, 4); */
  /* cairo_scale(cr, .25, .25); */

  /* erase the window */
  //cairo_set_source_rgb(cr, .5, .5, .5);
  //cairo_rectangle(cr, 0, 0, W, H);
  //cairo_fill(cr);

  /* transform for the image */
  cairo_translate(cr, W/2+ox, H/2+oy);
  cairo_rotate(cr, rotate);
  cairo_scale(cr, scale, scale);
  cairo_translate(cr, -WIDTH/2, -HEIGHT/2);

  cairo_set_source(cr, pattern);
  cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
  cairo_fill(cr);

  cairo_restore(cr);
}

int mouse_dx, mouse_dy;
int drag_rotate = 0;

void push(cairot_window_t window, int button, int X, int Y) {
  mouse_dx = X - ox;
  mouse_dy = Y - oy;
  int W, H; cairot_get_window_size(window, &W, &H);
  drag_rotate = X > W/2 && H > H/2;
}

void drag(cairot_window_t window, int button, int X, int Y) {
  if (drag_rotate) {
    int W, H; cairot_get_window_size(window, &W, &H);
    float dx = X - (W/2 + ox);
    float dy = Y - (H/2 + oy);
    scale = 2 * hypot(dx, dy) / WIDTH;
    rotate = atan2(dy, dx) - M_PI_4;
    printf("scale=%g, rotate=%g\n", scale, rotate);
  } else {
    ox = X - mouse_dx;
    oy = Y - mouse_dy;
  }
  cairot_redraw(window);
}

void release(cairot_window_t window, int button, int X, int Y) {
}

void keypress(cairot_window_t window, int keysym, const char* text) {
  cairo_filter_t f;
  cairo_extend_t e;
  switch (keysym) {
  case 'f': f = CAIRO_FILTER_FAST; goto SF;
  case 'd': f = CAIRO_FILTER_GOOD; goto SF;
  case 'b': f = CAIRO_FILTER_BEST; goto SF;
  case 'n': f = CAIRO_FILTER_NEAREST; goto SF;
  case 'l': f = CAIRO_FILTER_BILINEAR; goto SF;
  case 'g': f = CAIRO_FILTER_GAUSSIAN; goto SF;
  SF: cairo_pattern_set_filter(pattern, f);
    cairot_redraw(window);
    break;
  case '0': e = CAIRO_EXTEND_NONE; goto SE;
  case '1': e = CAIRO_EXTEND_PAD; goto SE;
  case '2': e = CAIRO_EXTEND_REPEAT; goto SE;
  case '3': e = CAIRO_EXTEND_REFLECT; goto SE;
  SE: cairo_pattern_set_extend(pattern, e);
    cairot_redraw(window);
    break;
  default:
    break;
  }
}

int main(int argc, char** argv)
{
  cairot_set_draw(draw);
  cairot_set_push(push);
  cairot_set_drag(drag);
  cairot_set_release(release);
  cairot_set_keypress(keypress);

  cairo_surface_t* image = makeimage();
  pattern = cairo_pattern_create_for_surface(image);
  cairot_window_t window = cairot_create_window(300, 300);
  cairot_set_window_name(window, argv[0]);
  cairot_set_window_state(window, CAIROT_STATE_MAPPED);
  return cairot_run();
}
