#include "cairot.h"
#include <cairo/cairo.h>
#include <pango/pangocairo.h>
#include <math.h>
#include <stdio.h>

const char* text = "www.cairographics.org";

void draw(cairot_window_t window)
{
  int W, H; cairot_get_window_size(window, &W, &H);
  cairo_t* cr = cairo_for(window);

  /* erase the window */
  cairo_set_source_rgb(cr, 1,1,1);
  cairo_rectangle(cr, 0, 0, W, H);
  cairo_fill(cr);

  PangoFontDescription *font_description = pango_font_description_new ();
  pango_font_description_set_family (font_description, "sans");
  //pango_font_description_set_weight (font_description, PANGO_WEIGHT_BOLD);
  pango_font_description_set_absolute_size (font_description, 32 * PANGO_SCALE);

  PangoLayout *layout = pango_cairo_create_layout (cr);
  pango_layout_set_font_description (layout, font_description);
  pango_layout_set_markup (layout, text, -1);
  pango_layout_set_width (layout, W * PANGO_SCALE);
  pango_layout_set_height (layout, H * PANGO_SCALE);
  pango_layout_set_alignment (layout, PANGO_ALIGN_CENTER);
  //pango_layout_set_justify (layout, TRUE);
  //pango_layout_set_single_paragraph_mode (layout, TRUE);
  //pango_layout_set_ellipsize (layout, PANGO_ELLIPSIZE_END);

  //cairo_save (cr);
  //cairo_rotate (cr, .1);
  cairo_set_source_rgb (cr, 0, 0, 0);
  cairo_move_to (cr, 0, 0); //10.0, 50.0);
  pango_cairo_show_layout (cr, layout);
  //cairo_restore (cr);

  g_object_unref (layout);
  pango_font_description_free (font_description);
}

int main(int argc, char** argv)
{
  if (argc == 2 && argv[1][0] != '-')
    text = argv[1];
  else if (argc > 1) {
    fprintf(stderr, "usage: %s <text>\n", argv[0]);
    return 1;
  }
  cairot_set_draw(draw);
  cairot_window_t window = cairot_create_window(300, 300);
  cairot_set_window_name(window, argv[0]);
  cairot_set_window_state(window, CAIROT_STATE_MAPPED);
  return cairot_run();
}
