#include "cairot.h"
#include <cairo/cairo.h>
#include <math.h>
#include <stdio.h>

void draw(cairot_window_t window)
{
  int W, H; cairot_get_window_size(window, &W, &H);
  cairo_t* cr = cairo_for(window);

  /* erase the window */
  cairo_set_source_rgb(cr, 1,1,1);
  cairo_rectangle(cr, 0, 0, W, H);
  cairo_fill(cr);

  /* draw something: */
  cairo_text_extents_t extents;

  const char *utf8 = "www.cairograpics.org";
  double x,y;

  cairo_select_font_face (cr, "Sans",
			  CAIRO_FONT_SLANT_NORMAL,
			  CAIRO_FONT_WEIGHT_NORMAL);

  cairo_set_font_size (cr, 52.0);
  cairo_text_extents (cr, utf8, &extents);
  x = (W-extents.width)/2 - extents.x_bearing;
  y = (H-extents.height)/2 - extents.y_bearing;

  cairo_move_to (cr, x, y);
  cairo_set_source_rgb(cr, 0,0,0);
  cairo_show_text (cr, utf8);

  /* draw helping lines */
  cairo_set_source_rgba (cr, 1, 0.2, 0.2, 0.6);
  cairo_set_line_width (cr, 6.0);
  cairo_arc (cr, x, y, 10.0, 0, 2*M_PI);
  cairo_fill (cr);
  cairo_move_to (cr, W/2.0, 0);
  cairo_rel_line_to (cr, 0, H);
  cairo_move_to (cr, 0, H/2.0);
  cairo_rel_line_to (cr, W, 0);
  cairo_stroke (cr);

}

int main(int argc, char** argv)
{
  cairot_set_draw(draw);
  cairot_window_t window = cairot_create_window(300, 300);
  cairot_set_window_name(window, argv[0]);
  cairot_set_window_state(window, CAIROT_STATE_MAPPED);
  return cairot_run();
}
